﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Drawing.Imaging;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;
using CommonUntil;
using ESocketCore;

namespace UIClient
{
    public partial class ClientFrm : SkinBase
    {
        public ClientFrm()
        {
            InitializeComponent();
        }

        private void btnChoose_Click(object sender, EventArgs e)
        {
            openFile.Filter = "All Files(*.*)|*.*";
            if (openFile.ShowDialog() == DialogResult.OK)
                txtPath.Text = openFile.FileName;
        }

        private Socket client;
        private SocketClient socketClient;
        private int bufferSize = Int16.MaxValue;
        private FileStream fs;
        private bool cancelFlag = false;

        private AutoResetEvent[] resetEvent;
        private AutoResetEvent resetEventEx = new AutoResetEvent(false);

        private void ClientFrm_Load(object sender, EventArgs e)
        {
            IPAddress localAddress = Until.GetLocalIP();
            socketClient = new SocketClient(localAddress, (int)PortCollection.ClientListenPort, bufferSize);
            socketClient.OnReceive = OnClientReceive;
            socketClient.StartListen();
        }


        private void OnClientReceive(SocketAsyncEventArgs e)
        {
            string flag = Encoding.ASCII.GetString(e.Buffer, 0, e.BytesTransferred);
            if (string.IsNullOrEmpty(flag)) return;

            Permission permission = (Permission)Enum.Parse(typeof(Permission), flag);
            txtLog.Log(string.Format("Receive CancelFlag as: {0}", flag));

            if (permission == Permission.Approve)
            {
                cancelFlag = true;
                txtLog.Log(string.Format("Start to send data to server side."));
            }

            if (permission == Permission.Deny)
            {
                cancelFlag = false;
                txtLog.Log(string.Format("Cancel to send data to server side."));
            }

            resetEventEx.Set();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            string ipAddress = txtAddress.Text;
            int result;

            if (string.IsNullOrEmpty(ipAddress)) return;

            IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(ipAddress), (int)PortCollection.ServerListenPort);
            client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            client.Connect(endPoint);
        }



        private void btnSend_Click(object sender, EventArgs e)
        {
            //use APM pattern to send byte array out
            BeginSend();
        }

        private void BeginSend()
        {
            Action action = new Action(SendBytes);
            action.BeginInvoke(new AsyncCallback((iar) =>
            {
                Action actionEnd = (Action)iar.AsyncState;
                actionEnd.EndInvoke(iar);

                txtLog.Log(string.Format("All are sent Complete! ")); 
                if (fs != null) fs.Close();
            }), action);
        }


        private void SendBytes()
        {
            fs = new FileStream(txtPath.Text, FileMode.Open);
            fs.Position = 0;
            long totalBytesLength = fs.Length;
            byte[] totalLenByte = Encoding.ASCII.GetBytes("[I=" + totalBytesLength + "]");
            client.Send(totalLenByte);

            txtLog.Log(string.Format("Waitting for Server action, pls wait..."));
            resetEventEx.WaitOne();

            if (!cancelFlag)
            {
                return;
            }

            byte[] byteToSend;

            //Calculate how many batches the file should be splitted.
            long totalBatch = 0;
            if (totalBytesLength % bufferSize == 0)
                totalBatch = totalBytesLength / bufferSize;
            else
                totalBatch = totalBytesLength / bufferSize + 1;

            //Sync the thread
            resetEvent = new AutoResetEvent[totalBatch];

            //Loop to send byte array out
            for (int i = 0; i < totalBatch; i++)
            {
                //reset stream position
                fs.Position = i * bufferSize;

                if (i == totalBatch - 1)
                {
                    long lastBatchLength = totalBytesLength - bufferSize * i;
                    byteToSend = new byte[lastBatchLength];
                    fs.Read(byteToSend, 0, (int)lastBatchLength);
                }
                else
                {
                    byteToSend = new byte[bufferSize];
                    fs.Read(byteToSend, 0, bufferSize);

                }

                //avoid closure 
                int j = i;  

                //init the resetEvent to block thread.that is , main thread and child thread will run together, 
                // when  WaitOne function got the set signal, it will go on to run.
                resetEvent[j] = new AutoResetEvent(false);
                
                client.BeginSend(byteToSend, 0, byteToSend.Length, SocketFlags.None, (iar) =>
                {
                    resetEvent[j].Set();
                    client.EndSend(iar);
                    txtLog.Log(String.Format("{0} batch was sent complete.", j));
                    
                }, client);

                txtLog.Log(String.Format("{0} batch,Send {1} bytes to server side.", i, byteToSend.Length));
                Array.Clear(byteToSend, 0, byteToSend.Length);

                //waitting for set signal.
                resetEvent[j].WaitOne();
            }

        }

        //Object to Byte[]
        private byte[] ObjectToByteArray(object obj)
        {
            if (obj == null)
                return null;
            BinaryFormatter binaryFormatter = new BinaryFormatter();
            MemoryStream memoryStream = new MemoryStream();
            binaryFormatter.Serialize(memoryStream, obj);
            return memoryStream.ToArray();
        }
    }
}
