﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ESocketCore;
using System.Net;
using System.Net.Sockets;
using System.IO;
using CommonUntil;
using System.Text.RegularExpressions;

namespace UIServer
{
    public partial class MainFrm : SkinBase
    {
        public MainFrm()
        {
            InitializeComponent();
        }

        private SocketServer socketServer;
        private int batchCount = 0;
        private string file;

        private long totalBytes = 0;
        private long curBytes = 0;

        private void btnStart_Click(object sender, EventArgs e)
        {
            string ipAddress = txtAddress.Text;
            if (string.IsNullOrEmpty(ipAddress))
                return;

            if (socketServer == null)
            {
                socketServer = new SocketServer(IPAddress.Parse(ipAddress), (int)PortCollection.ServerListenPort, 10, 10, Int16.MaxValue);
                socketServer.OnStarted = OnStarted;
                socketServer.OnStopped = OnStopped;
                socketServer.OnConnected = OnConnected;
                socketServer.OnDisconnect = OnDisconnected;
                socketServer.OnReceive = OnReceive;
            }
            socketServer.Start();
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            socketServer.Stop();
        }

        private void OnStarted(Socket socket)
        {
            txtLog.Log(string.Format("Server started...")); 
        }

        private void OnStopped()
        {
            txtLog.Log("Server Stopped.");
        }

        private void OnConnected(Socket socket)
        {
            if (socket.RemoteEndPoint != null)
                txtLog.Log(string.Format("Client {0} Connected to Server...", socket.RemoteEndPoint.ToString()));
        }

        private void OnDisconnected(SocketAsyncEventArgs e)
        {
            if (e.RemoteEndPoint != null)
                txtLog.Log(string.Format("Client {0} disconnected.", e.RemoteEndPoint.ToString()));
        }

        private void OnReceive(SocketAsyncEventArgs e)
        {
            batchCount++;
            if (batchCount == 1)
            {
                FileOperation();

                string lenStr = Encoding.ASCII.GetString(e.Buffer, e.Offset, e.BytesTransferred);
                txtLog.Log(string.Format("Receive {0} batch data, {1} bytes : {2}", batchCount, e.BytesTransferred, lenStr));
                
                GetTotalBytes(lenStr);

                pbBar.UIInvoke(() => { pbBar.Value = 0; });
            }
            else
            {
                txtLog.Log(string.Format("Receive {0} batch data, {1} bytes", batchCount, e.BytesTransferred));
                curBytes += e.BytesTransferred;
            }

            if (batchCount > 1)
            {
                using (FileStream fileStream = new FileStream(file, FileMode.Append))
                {
                    fileStream.Write(e.Buffer, e.Offset, e.BytesTransferred);
                    fileStream.Flush();
                }
            }

            pbBar.UIInvoke(() => { pbBar.Value = (int)(curBytes * 100 / totalBytes); });

            if (curBytes == totalBytes)
            {

                pbBar.UIInvoke(() => { pbBar.Value = 100; });

                txtLog.Log(string.Format("Totally Receive {0} bytes. Receive complete! ", curBytes));

                ResetState();
            }
        }

        private void ResetState()
        {
            curBytes = 0;
            totalBytes = 0;
            batchCount = 0;
        }

        private void GetTotalBytes(string content)
        {
            Match match = Regex.Match(content, @"[0-9]+");
            totalBytes = match.Success ? Int32.Parse(match.Value) : 0;
        }

        private void FileOperation()
        {
            this.Invoke((Action)(() =>
            {
                SaveFileDialog saveFileDlg = new SaveFileDialog();
                saveFileDlg.Filter = "All Files(*.*)|*.*";
                if (saveFileDlg.ShowDialog() == DialogResult.OK)
                {
                    file = saveFileDlg.FileName;
                    socketServer.Send(Encoding.ASCII.GetBytes(((int)Permission.Approve).ToString()));  //"1" --Can go on sending file
                    txtLog.Log(string.Format("Start accept file data from Client side..."));
                }
                else
                {
                    socketServer.Send(Encoding.ASCII.GetBytes(((int)Permission.Deny).ToString()));  //"0" --Cancel sending file
                    txtLog.Log(string.Format("Stop accept file data from Client side..."));
                    return;
                }
            }));
        }

       
    }
}
